//    Demo 6d: Student.java

//  A simple Student record type implementing the LinearlyOrdered interface.
//
//                by Charles W. Neville, 1998
//             Copyright Charles W. Neville, 1998
//
//  We have to redo the Student class from Demo 4, RecordAccess.java, so it is 
//  public and implements the LinearlyOrdered interface.
//
//  The LinearlyOrdered interface requires classes to implement methods gt, lt 
//  and eq.  The Student class from Demo 4 already implements those methods, so 
//  all we have to add is an announcement to the Java world that class Name 
//  implements interface LinearlyOrdered.

import java.io.*;
import LinearlyOrdered;
import Name;

// student record  -- a NESTED RECORD data type
public class Student implements LinearlyOrdered {

   Name   name;
   int    age;
   double gpa;

   // read a Student record from a BufferedReader
   //
   // returns the record read -- returns null f there are no more records,
   // throws IOExpection and NumberFormatException
   //
   // input assumptions:
   //
   // (1) This method assumes each field of the record in the underlying file
   // is contained on a SEPARATE LINE, with NO LEADING OR TRAILING SPACES, as 
   // in the example below:
   //
   //  Joe
   //  M
   //  Bflspk
   //  22
   //  3.14159
   //
   // (2) This method assumes the BufferedReader is positioned at the beginning
   // of the line containing the first name field of the current record, Joe in
   // the example above.
   //
   // (3) When using this method to read student records to the end-of-file, be 
   // sure the LAST LINE of the input file contains a zip code, and that there 
   // are NO BLANK LINES at the end of the file.
   //
   // unless these assumptions are adhered to rigorously, this method will not 
   // work correctly.  (It is highly sensitive to mistakes in the format of the 
   // input file.)
   public static Student readStudent(BufferedReader br) throws IOException {

      Student s = new Student();

      s.name = Name.readName(br);
      if (s.name == null) return null;  // end of file -- return null

     s.age  = (Integer.valueOf(br.readLine()).intValue());  // throws 
                  // NumberFormatException -- could equally well use 
                  // Integer.parseInt.
     s.gpa  = (Double.valueOf(br.readLine()).doubleValue());  // throws 
                  // NumberFormatException -- there is no Double.parseDouble.

     return s;

   } // readStudent

   // write a Student record to standard output
   public void writeStudent() {

        name.writeName();  // write the whole name subrecord
        System.out.println(age);
        System.out.println(gpa);
   }
 
   // is this Student record > Student record b?  Compare with name first, then 
   // age, then GPA.  gt(b) returns true if this record  > record b, false 
   // otherwise.  Java silliness requires us to parametrize gt as 
   // gt(LinearlyOrdered b) rather than gt(Student b).
   public boolean gt(LinearlyOrdered b) {

      Student c = (Student) b;  // more Java silliness, cast parameter b 
                                // to a Student object

      // because of return statements, can just use bunch of if's, don't need
      // if -- else if -- else.
      if (this.name.gt(c.name)) return true;  // this says,
           // if( this.name > c.name) return true, but have to use gt
           // method in place of > operator.
      if (this.name.lt(c.name)) return false;  // this says,
           // if( this.name < c.name) return false, but have to use gt
           // method in place of < operator.
      if (this.age > c.age) return true;
      if (this.age < c.age) return false;
      if (this.gpa > c.gpa) return true;
      if (this.gpa < c.gpa) return false;

      return false;  // two records are equal in this case.

   } // gt

   // is this Student record < Student record b?  Compare with name first, then 
   // age, then GPA.  lt(b) returns true if this record  < record b, false 
   // otherwise.  Java silliness requires us to parametrize lt as 
   // lt(LinearlyOrdered b) rather than gt(Student b).
   public boolean lt(LinearlyOrdered b) {

      Student c = (Student) b;  // more Java silliness, cast parameter b 
                                // to a Student object

      // because of return statements, can just use bunch of if's, don't need
      // if -- else if -- else.
      if (this.name.lt(c.name)) return true;  // this says,
           // if( this.name < c.name) return true, but have to use lt
           // method in place of < operator.
      if (this.name.gt(c.name)) return false;  // this says,
           // if( this.name < c.name) return false, but have to use lt
           // method in place of < operator.
      if (this.age < c.age) return true;
      if (this.age > c.age) return false;
      if (this.gpa < c.gpa) return true;
      if (this.gpa > c.gpa) return false;

      return false;  // two records are equal in this case.

   } // gt

   // is this Student record = Student record b?  eq(b) returns true if 
   // this record  = record b, false otherwise.  Java silliness requires us to 
   // parametrize eq as eq(LinearlyOrdered b) rather than eq(Student b).
   public boolean eq(LinearlyOrdered b) {

      return !gt(b) && !lt(b);

   } // eq 

}  // class Student

